
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_control
  !----------------------------------------------------------------
  ! module to initialze and control the frequency calculation
  ! 
  !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  ! re-written by J. Neugebauer, Uni Erlangen, 2003,
  ! and modified by M. Reiher, Uni Erlangen, June 2003
  ! and by C. Herrmann, Uni Bonn, February 2005
  ! and by C. Herrmann, Uni Jena, April-June 2005
  !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  ! uses parts of subroutines by C. Kind and M. Reiher, 
  ! Uni Erlangen, 2000-2001
  !----------------------------------------------------------------
  use f90_kind
  use snf_error, only : serror
  use snf_turbomole,only : type_tm_ctrl
  use snf_molecule, only: type_atom
  use snf_dalton,   only : type_dal_ctrl
  use snf_adf,      only : type_adf_ctrl
  use snf_gauss,    only : type_gauss_ctrl
  
  private
  public :: read_control, write_control, copy_tm_to_ctrl,copy_ctrl_to_tm
  public :: type_ctrl,print_ctrl,check_ctrl,init_ctrl,destroy_control
  public :: get_energy,get_energy_mp2,copy_ctrl_to_dal,get_spin
  public :: get_dalenergy,get_dalescf,read_control2, copy_ctrl_to_adf
  public :: copy_ctrl_to_gauss

  !----------------------------------------------------------------
  ! progno numbers: 
  ! 
  ! 1 - Turbomole
  ! 2 - Dalton
  ! 3 - Gaussian
  ! 4 - ADF
  !----------------------------------------------------------------

  type type_ctrl
     character(len=132) :: title     ! title of calculation
     type(type_tm_ctrl),pointer :: tm_ctrl
     type(type_dal_ctrl),pointer :: dal_ctrl
     type(type_adf_ctrl),pointer :: adf_ctrl
     type(type_gauss_ctrl),pointer :: gauss_ctrl
     integer :: ramanew              ! no. of raman exc. wavelengths
     integer :: vroaew               ! no. of VROA exc. wavelengths
     logical :: sumdeg               ! sum intensities for degenerate modes
     logical :: maxsym               ! use symmetry to determine redundant
                                     ! distortions for non-redundant atoms
     integer :: version, &           ! version number  
          &     nvar, &              ! no. of variables = no. of d.o.f.
          &     progno, &            ! no. which determines the program for single points
          &     calcno, &            ! no. which determines the type of calculation
          &     numderiv             ! flag for first-derivative 
                                     ! (1: 3-point formula; 2: 5-point formula)
     real(double) :: cstep           ! step size (in cartesian coordinates) in bohr
     real(double) :: freqscale       ! scaling factor for frequencies
     character(len=3) :: symmetry    ! schoenfliess symbol of point group
     character(len=3) :: isosym      ! schoenfliess symbol of point group 
                                     ! after selection of isotopes
     character(len=5) :: dtype       ! type of numerical derivatives
                                     ! (centr=central, fward=forward, bward=backward)
     character(len=80) :: restart    ! filename for restart file
     character(len=255):: eligible
     character(len=80) :: control    ! filename for snf_control file
     character(len=9) :: ramanptype  ! type of data to be plotted in Raman spectrum
     character(len=9) :: vroaptype   ! type of data to be plotted in Raman spectrum
     character(len=7) :: peaktype    ! Gauss or Lorentz type for peak profile
!MR: The following line might be moved to a different type/place:
     character(len=7) :: dal_func    ! the functional for a DALTON DFT run
!
     real(double) :: minfreq,maxfreq,width,theta   ! theta = scattering angle
     integer :: npoint
     real(double) :: temp,press,elec_energy,spin
     logical :: print_gaussian,&     ! print gaussian fake output
          &     print_mopac, &       ! print mopac fake output
          &     print_mopxmol, &     ! print mopac fake output for use in xmol
          &     print_xmol, &        ! print xmol XYZ input file
          &     print_symmetry, &    ! print detailed symmetry-information
          &     print_irred_at, &    ! print infos about irreducible atoms
          &     print_irred_dist,&   ! print infos about irreducible distortions
          &     print_cioslowski, &  ! print cioslowski charge decomposition
          &     print_force, &       ! print force constant matrix
          &     print_hessian, &     ! print cartesian hessian
          &     print_dip, &         ! print cartesian dipol derivates
          &     print_pol, &         ! print cartesian polarizability derivates
          &     print_gaten, &       ! print cartesian G and A tensor derivates
          &     print_rcs, &         ! print raman cross sections
          &     print_vroa, &        ! print VROA results
          &     print_gten_lao, &    ! print cartesian g tensor (LAO) derivates
          &     print_gten_ao, &     ! print cartesian g tensor (LAO) derivates
          &     print_aten, &        ! print cartesian A tensor derivates
          &     print_transrot, &    ! print transl. and rot. contributions
          &     print_zpe, &         ! print  zeropoint energy 
          &     tmpdirclean, &       ! remove temporary directories after calculation
          &     logdirclean, &       ! remove logfiles after calculation
          &     print_thermo         ! print thermodynamics
     real(double) :: xmolsc1         ! scale factor for xmol output
     real(double),pointer :: dynfreq(:) ! frequencies for the calculation
                                        ! of dynamic polarizabilities
     integer,pointer :: selectmode(:)! active mode flag,(intensities-only program mode)
     integer :: nummodes             ! number of modes (intensities-only program mode)
     integer :: disptype             ! type of coordinate displacement along normal modes 
     real(double),pointer :: freqin(:),nmodein(:,:)
  end type type_ctrl

  !************************************************************************

contains

  !************************************************************************

  subroutine init_ctrl(ctrl)
    implicit none
    !----------------------------------------------------------------
    ! initialize default ctrl values
    ! 
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    ! re-written by J. Neugebauer, Uni Erlangen, 2003
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by J. Neugebauer, Uni Erlangen, 2001, 2002
    ! modified by Markus Reiher, Uni Erlangen, Aug./Nov. 2001, June 2003
    ! modified by C. Herrmann, Uni Bonn, 02/2005
    ! modified by C. Herrmann, Uni Jena, 04-06/2005
    !----------------------------------------------------------------
    include '../snfdc/VERSION.h'
    type(type_ctrl),intent(out) :: ctrl
    ctrl%title= 'snf calculation'
    nullify(ctrl%tm_ctrl)
    nullify(ctrl%dal_ctrl)
    nullify(ctrl%adf_ctrl)
    nullify(ctrl%gauss_ctrl)
    nullify(ctrl%dynfreq)
    ctrl%version= ivers
    ctrl%ramanew=0
    ctrl%vroaew=0
    ctrl%sumdeg=.true.
    ctrl%symmetry='c1 '
    ctrl%isosym='def'
    ctrl%ramanptype='(ractivi)'
    ctrl%vroaptype='(default)'
    ctrl%peaktype='gauss  '
    ctrl%numderiv=1
    ctrl%dtype='centr'
    ctrl%control='snf_control'
    ctrl%restart='restart'
    ctrl%eligible='$CIPROC/ELIGIBLE'
    ctrl%cstep=1.0d-2
    ctrl%progno=1
    ctrl%calcno=1
    ctrl%xmolsc1=2.0d0
    ctrl%theta= 90.0d0
    ctrl%minfreq=200
    ctrl%maxfreq=3500
    ctrl%width=20
    ctrl%npoint=2000
    ctrl%freqscale=1.0d0
    ctrl%temp=273.15d0+25.0d0
    ctrl%press=101325.0d0
!MR: The following line might be moved to a different type/place:
!    (see type declaration)
    ctrl%dal_func='       '
!
    ctrl%print_gaussian=.true.
    ctrl%print_mopac=.false.
    ctrl%print_mopxmol=.false.
    ctrl%print_xmol=.false.
    ctrl%print_symmetry=.false.
    ctrl%print_irred_at=.false.
    ctrl%print_irred_dist=.false.
    ctrl%print_cioslowski=.true.
    ctrl%print_force=.false.
    ctrl%print_hessian=.false.
    ctrl%print_dip=.false.
    ctrl%print_rcs=.false.
    ctrl%print_vroa=.false.
    ctrl%print_pol=.false.
    ctrl%print_gaten=.false.
    ctrl%print_transrot=.true.
    ctrl%print_zpe=.true.
    ctrl%print_thermo=.true.
    ctrl%logdirclean=.false.
    ctrl%tmpdirclean=.false.
    ctrl%maxsym=.true.
    ctrl%elec_energy=0.0d0
    ctrl%spin=0.0d0
    nullify(ctrl%selectmode)
    nullify(ctrl%freqin)
    nullify(ctrl%nmodein)
    ctrl%nummodes = 0
    ctrl%disptype = 1
  end subroutine init_ctrl

  !************************************************************************

  subroutine read_control(ctrl,selatom,massatom)
    !----------------------------------------------------------------
    ! read all avaible data from snf_control
    ! 
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    ! re-written by J. Neugebauer, Uni Erlangen, 2003 
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by Markus Reiher, Uni Erlangen, Aug./Nov. 2001, June 2003
    ! modified by J. Neugebauer, Uni Erlangen, 2001 
    ! modified by C. Herrmann, Uni Jena, 2005
    !----------------------------------------------------------------
    use snf_turbomole, only : read_tm_ctrl, init_tm_ctrl
    use snf_tools, only: string2integer,string2real,getdfreq,split
    implicit none
    type(type_ctrl),intent(inout) :: ctrl
    character(len=132)          :: buf
    real(double),pointer :: massatom(:)
    integer, pointer :: selatom(:)
    integer :: ucf,nchange
    logical :: ex,foundmaxsym
    integer :: i,j, mxitem,natoms
    parameter(mxitem =10)
    character(len=132) :: item(mxitem)
    ! --------- read control file completely -------------------------
    ucf = 12
    buf='  '
    inquire(file=ctrl%control,exist=ex)
    if(.not. ex) then
       call serror('read_control',2,'snf_control does not exist.##&
         &If you want to evaluate restart-files from former##&
         &versions of snf, please first run snf_define to get##&
         &the snf_control file, and then run snf again.')
    end if

    open(unit=ucf,file=ctrl%control,status='unknown')

    nchange=0
    do while(index(buf,'$end').eq.0)
       read(ucf,'(a)') buf
       if(index(buf,'$changemass').ne.0) nchange=nchange+1
    end do
    rewind(ucf)
    if(nchange.ge.1) then
       allocate(selatom(nchange),massatom(nchange))
       write(*,*) '>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<'
       write(*,*) ' Found modified isotopes for the following atoms:'
       write(*,*) 
       write(*,*) ' No.    atomic mass [amu]:'
       write(*,*) 
    end if
    nchange=1
    foundmaxsym=.false.
    buf=''

    do while(index(buf,'$end').eq.0)
       read(ucf,'(a)') buf
       if(index(buf,'$title ').ne.0) then
         read(ucf,'(a)') buf
         ctrl%title=trim(buf)
       else if( index(buf,'$calcno').ne.0) then
         ctrl%calcno=string2integer(buf)
       else if( index(buf,'$progno').ne.0) then
          ctrl%progno=string2integer(buf)
          if(ctrl%progno.eq.1) then
            if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
            allocate(ctrl%tm_ctrl)
            call init_tm_ctrl(ctrl%tm_ctrl)
            call read_tm_ctrl(ctrl%tm_ctrl)
            call copy_tm_to_ctrl(ctrl)
          else if(ctrl%progno.eq.2) then
            if(associated(ctrl%dal_ctrl)) deallocate(ctrl%dal_ctrl)
            allocate(ctrl%dal_ctrl)
!            call init_dal_ctrl(ctrl%dal_ctrl)
!            call read_dal_ctrl(ctrl%tm_ctrl)
          else if(ctrl%progno.eq.3) then
            if(associated(ctrl%gauss_ctrl)) deallocate(ctrl%gauss_ctrl)
            allocate(ctrl%gauss_ctrl)
!            call init_gauss_ctrl(ctrl%gaussctrl)
!            call read_gauss_ctrl(ctrl%tm_ctrl)
          else if(ctrl%progno.eq.4) then
            if(associated(ctrl%adf_ctrl)) deallocate(ctrl%adf_ctrl)
            allocate(ctrl%adf_ctrl)
!            call init_adf_ctrl(ctrl%adfctrl)
!            call read_adf_ctrl(ctrl%tm_ctrl)
          else
            write(*,*) 'PROGNO = ', ctrl%progno 
            call serror('read_ctrl',2,'Found invalid progno number',6)
          end if
       else if( index(buf,'$symmetry ').ne.0) then
          ctrl%symmetry=trim(buf(11:))
       else if( index(buf,'$isosym ').ne.0) then
          ctrl%isosym=trim(buf(9:))
       else if( index(buf,'$dtype ').ne.0) then
          ctrl%dtype=trim(buf(8:))
       else if( index(buf,'$rptype ').ne.0) then
          ctrl%ramanptype=trim(buf(9:))
       else if( index(buf,'$vroaptype ').ne.0) then
          ctrl%vroaptype=trim(buf(12:))
       else if( index(buf,'$peaktype ').ne.0) then
          ctrl%peaktype=trim(buf(11:))
          if(ctrl%peaktype.ne.'lorentz') ctrl%peaktype='gauss  '
       else if( index(buf,'$tmpdirclean').ne.0) then
          ctrl%tmpdirclean=.true.
       else if( index(buf,'$logdirclean').ne.0) then
          ctrl%logdirclean=.true.
       else if( index(buf,'$ramanew').ne.0) then
          ctrl%ramanew=string2integer(buf)
       else if( index(buf,'$vroaew').ne.0) then
          ctrl%vroaew=string2integer(buf)
       else if( index(buf,'$dalmw').ne.0.and.associated(ctrl%dal_ctrl)) then
          ctrl%dal_ctrl%dalmw=string2integer(buf)
       else if( index(buf,'$dynfreq').ne.0) then
          if(ctrl%ramanew.gt.0) then
            call getdfreq(buf(10:),ctrl%dynfreq)
          else if (ctrl%vroaew.eq.0) then
            call serror('read_control',2,'Found $dynfreq but $ramanew = 0')
          end if
          if(ctrl%vroaew.gt.0.and..not.ctrl%ramanew.gt.0) then
            call getdfreq(buf(10:),ctrl%dynfreq)
          else if (ctrl%ramanew.eq.0) then
            call serror('read_control',2,'Found $dynfreq but $vroaew = 0')
          end if
       else if( index(buf,'$cstep').ne.0) then
          ctrl%cstep=string2real(buf)
       else if( index(buf,'$numderiv').ne.0) then
          ctrl%numderiv=string2integer(buf)
       else if( index(buf,'$restart').ne.0) then
          ctrl%restart=trim(buf(index(buf,'=')+1:))
       else if( index(buf,'$minfreq').ne.0) then
          ctrl%minfreq=string2real(buf(index(buf,'$minfreq')+8:))
       else if( index(buf,'$maxfreq').ne.0) then
          ctrl%maxfreq=string2real(buf(index(buf,'$maxfreq')+8:))
       else if( index(buf,'$width').ne.0) then
          ctrl%width=string2real(buf(index(buf,'$width')+6:))
       else if( index(buf,'$freqscale').ne.0) then
          ctrl%freqscale=string2real(buf(index(buf,'$freqscale')+10:))
       else if( index(buf,'$npoint').ne.0) then
          ctrl%npoint=string2integer(buf(index(buf,'$npoint')+7:))
       else if( index(buf,'$temp').ne.0) then
          ctrl%temp=string2real(buf(index(buf,'$temp')+5:))
       else if( index(buf,'$theta').ne.0) then
          ctrl%theta=string2real(buf(index(buf,'$theta')+6:))
       else if( index(buf,'$press').ne.0) then
          ctrl%press=string2real(buf(index(buf,'$press')+6:))
       else if( index(buf,'$xmolsc1').ne.0) then
          ctrl%xmolsc1=string2real(buf(index(buf,'$xmolsc1')+8:))
       else if( index(buf,'$print_gaussian').ne.0) then
          if(index(buf,'on' ).ne.0) ctrl%print_gaussian = .true.
          if(index(buf,'off').ne.0) ctrl%print_gaussian  = .false.
       else if( index(buf,'$print_mopac').ne.0) then      
          if(index(buf,'on' ).ne.0) ctrl%print_mopac = .true.
          if(index(buf,'off').ne.0) ctrl%print_mopac  = .false.
       else if( index(buf,'$print_mopxmol').ne.0) then      
          if(index(buf,'on' ).ne.0) ctrl%print_mopxmol = .true.
          if(index(buf,'off').ne.0) ctrl%print_mopxmol  = .false.
       else if( index(buf,'$print_xmol').ne.0) then      
          if(index(buf,'on' ).ne.0) ctrl%print_xmol = .true.
          if(index(buf,'off').ne.0) ctrl%print_xmol  = .false.
       else if( index(buf,'$print_symmetry').ne.0) then   
          if(index(buf,'on' ).ne.0) ctrl%print_symmetry = .true.
          if(index(buf,'off').ne.0) ctrl%print_symmetry  = .false.
       else if( index(buf,'$print_irred_at').ne.0) then   
          if(index(buf,'on' ).ne.0) ctrl%print_irred_at = .true.
          if(index(buf,'off').ne.0) ctrl%print_irred_at = .false.
       else if( index(buf,'$print_irred_dist').ne.0) then   
          if(index(buf,'on' ).ne.0) ctrl%print_irred_dist = .true.
          if(index(buf,'off').ne.0) ctrl%print_irred_dist = .false.
       else if( index(buf,'$print_cioslowski').ne.0) then 
          if(index(buf,'on' ).ne.0) ctrl%print_cioslowski = .true.
          if(index(buf,'off').ne.0) ctrl%print_cioslowski = .false.
       else if( index(buf,'$print_force').ne.0) then      
          if(index(buf,'on' ).ne.0) ctrl%print_force = .true.
          if(index(buf,'off').ne.0) ctrl%print_force = .false.
       else if( index(buf,'$print_hessian').ne.0) then    
          if(index(buf,'on' ).ne.0) ctrl%print_hessian = .true.
          if(index(buf,'off').ne.0) ctrl%print_hessian = .false.
       else if( index(buf,'$print_dip').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_dip = .true.
          if(index(buf,'off').ne.0) ctrl%print_dip = .false.
       else if( index(buf,'$sumdeg').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%sumdeg = .true.
          if(index(buf,'off').ne.0) ctrl%sumdeg = .false.
       else if( index(buf,'$maxsym').ne.0) then        
          foundmaxsym=.true.
          if(index(buf,'on' ).ne.0) ctrl%maxsym = .true.
          if(index(buf,'off').ne.0) ctrl%maxsym = .false.
       else if( index(buf,'$print_pol').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_pol = .true.
          if(index(buf,'off').ne.0) ctrl%print_pol = .false.
       else if( index(buf,'$print_gaten').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_gaten = .true.
          if(index(buf,'off').ne.0) ctrl%print_gaten = .false.
       else if( index(buf,'$print_rcs').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_rcs = .true.
          if(index(buf,'off').ne.0) ctrl%print_rcs = .false.
       else if( index(buf,'$print_vroa').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_vroa = .true.
          if(index(buf,'off').ne.0) ctrl%print_vroa = .false.
       else if( index(buf,'$print_transrot').ne.0) then   
          if(index(buf,'on' ).ne.0) ctrl%print_transrot = .true.
          if(index(buf,'off').ne.0) ctrl%print_transrot = .false.
       else if( index(buf,'$print_zpe').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_zpe = .true.
          if(index(buf,'off').ne.0) ctrl%print_zpe = .false.
       else if( index(buf,'$print_thermo').ne.0) then        
          if(index(buf,'on' ).ne.0) ctrl%print_thermo = .true.
          if(index(buf,'off').ne.0) ctrl%print_thermo = .false.
       else if( index(buf,'$changemass').ne.0) then        
          read(buf(12:),*) selatom(nchange),massatom(nchange)
          write(*,'(i5,f24.16)') selatom(nchange),massatom(nchange)
          if(nchange.eq.ubound(selatom,1)) &
               write(*,*) '>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<'
          nchange=nchange+1
       else if( index(buf,'$total electronic energy ').ne.0) then 
          ctrl%elec_energy=string2real(buf(index(buf,&
            &'$total electronic energy ')+25:))
       else if( index(buf,'$total spin ').ne.0) then
          ctrl%spin=string2real(buf(index(buf,& 
            &'$total spin ')+10:)) 
       endif
    enddo
    if(.not.foundmaxsym) ctrl%maxsym=.false.
    close(ucf)    
  end subroutine read_control

  !************************************************************************
  subroutine read_control2(ctrl,natoms)
    !----------------------------------------------------------------
    ! read normal modes and frequencies from snf_control
    !
    ! written by C. Herrmann, Uni Bonn, 02/2005 
    !----------------------------------------------------------------
    use snf_turbomole, only : read_tm_ctrl, init_tm_ctrl
    use snf_tools, only: string2integer,string2real,getdfreq,split
    implicit none
    type(type_ctrl),intent(inout) :: ctrl
    character(len=132)          :: buf
    real(double),pointer :: massatom(:)
    integer, pointer :: selatom(:)
    integer :: ucf,nchange
    logical :: ex,foundmaxsym
    integer :: i,j, mxitem,natoms,k
    parameter(mxitem =10)
    character(len=132) :: item(mxitem)
    ! --------- read control file completely -------------------------
    ucf = 12
    buf='  '
    inquire(file=ctrl%control,exist=ex)
    if(.not. ex) then
       call serror('read_control2',2,'snf_control does not exist.##&
         &If you want to evaluate restart-files from former##&
         &versions of snf, please first run snf_define to get##&
         &the snf_control file, and then run snf again.')
    end if
    open(unit=ucf,file=ctrl%control,status='unknown')
    do while(index(buf,'$end').eq.0)
       read(ucf,'(a)') buf
       if( index(buf,'$nummodes ').ne.0) then
          ctrl%nummodes=string2real(buf(10:132))
          do j = 1,ctrl%nummodes
            read(ucf,'(a)') buf
            call split (buf,item,mxitem)
            ctrl%freqin(j)=string2real(item(1))
            ctrl%nmodein(j,1) = string2real(item(2))
            ctrl%nmodein(j,2) = string2real(item(3))
            ctrl%nmodein(j,3) = string2real(item(4))
            do i=2,natoms
              read(ucf,'(a)') buf
              call split (buf,item,mxitem)
              ctrl%nmodein(j,(i-1)*3+1) = string2real(item(1))
              ctrl%nmodein(j,(i-1)*3+2) = string2real(item(2))
              ctrl%nmodein(j,(i-1)*3+3) = string2real(item(3))
            end do
            ctrl%selectmode(j) = 1
          end do
       end if
     end do
     close(ucf)
  end subroutine read_control2
  !************************************************************************

  subroutine write_control(ctrl,molecule,unit,allmodes)
    implicit none
    !-----------------------------------------------------------------
    ! write all available data to snf_control
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by Markus Reiher, Uni Erlangen, Aug./Nov. 2001, June 2003
    ! modified and extended by J. Neugebauer, Uni Erlangen, 2001, 2002
    ! modified by C. Herrmann, Uni Bonn, February 2005
    !-----------------------------------------------------------------
    type(type_ctrl),intent(in) :: ctrl
    type(type_atom),pointer    :: molecule(:)
    integer :: natoms
    integer,optional,intent(in) :: unit
    integer :: ucf,out,i,j,selectmode,allmodes
    ! --------- read whole control file  -------------------------
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    ucf = 43
    open(unit=ucf,file=ctrl%control,status='unknown')
    write(ucf,'(''$snf_control'')') 
    write(ucf,'(''$title'')')
    write(ucf,'(a)')trim(ctrl%title)
    write(ucf,'(a,i10)') '$progno ',ctrl%progno
    write(ucf,'(a,i10)') '$calcno ',ctrl%calcno
    write(ucf,'(''$cstep    '',f12.3)') ctrl%cstep
    write(ucf,'(''$symmetry '',a3)') ctrl%symmetry
    if(ctrl%isosym.ne.'def') &
         write(ucf,'(''$isosym '',a3)') ctrl%isosym
    if(ctrl%dtype.ne.'centr') &
         write(ucf,'(''$dtype '',a5)') ctrl%dtype
    write(ucf,'(''$rptype '',a9)') ctrl%ramanptype
    write(ucf,'(''$vroaptype '',a9)') ctrl%vroaptype
    write(ucf,'(''$peaktype '',a7)') ctrl%peaktype
    write(ucf,'(''$restart ='',a)') trim(ctrl%restart)
    write(ucf,'(''$minfreq '',f10.2)') ctrl%minfreq
    write(ucf,'(''$maxfreq '',f10.2)') ctrl%maxfreq
    write(ucf,'(''$width   '',f10.2)') ctrl%width
    write(ucf,'(''$freqscale '',f24.12)') ctrl%freqscale
    if(ctrl%print_xmol) write(ucf,'(''$xmolsc1 '',f14.6)') ctrl%xmolsc1
    write(ucf,'(''$npoint  '',i7)') ctrl%npoint
    write(ucf,'(''$temp    '',f10.2)') ctrl%temp
    write(ucf,'(''$theta   '',f10.3)') ctrl%theta
    write(ucf,'(''$press   '',f10.2)') ctrl%press
    if(ctrl%print_gaussian) then
       write(ucf,'(''$print_gaussian on'')')
    else                                     
       write(ucf,'(''$print_gaussian off'')')
    endif
    if(ctrl%print_mopac) then
       write(ucf,'(''$print_mopac on'')')
    else
       write(ucf,'(''$print_mopac off'')')
    endif
    if(ctrl%print_mopxmol) then
       write(ucf,'(''$print_mopxmol on'')')
    else
       write(ucf,'(''$print_mopxmol off'')')
    endif
    if(ctrl%print_xmol) then
       write(ucf,'(''$print_xmol on'')')
    else
       write(ucf,'(''$print_xmol off'')')
    endif
    if(ctrl%print_symmetry) then
       write(ucf,'(''$print_symmetry on'')')
    else
       write(ucf,'(''$print_symmetry off'')')
    endif
    if(ctrl%print_irred_at) then
       write(ucf,'(''$print_irred_at on'')')
    else
       write(ucf,'(''$print_irred_at off'')')
    endif
    if(ctrl%print_irred_dist) then
       write(ucf,'(''$print_irred_dist on'')')
    else
       write(ucf,'(''$print_irred_dist off'')')
    endif
    if(ctrl%print_cioslowski) then
       write(ucf,'(''$print_cioslowski on'')')
    else
       write(ucf,'(''$print_cioslowski off'')')
    endif
    if(ctrl%print_force) then
       write(ucf,'(''$print_force on'')')
    else
       write(ucf,'(''$print_force off'')')
    endif
    if(ctrl%print_hessian) then
       write(ucf,'(''$print_hessian on'')')
    else                                    
       write(ucf,'(''$print_hessian off'')')
    endif
    if(ctrl%print_dip) then
       write(ucf,'(''$print_dip on'')')
    else                                
       write(ucf,'(''$print_dip off'')')
    endif
    if(ctrl%maxsym) then
       write(ucf,'(''$maxsym on'')')
    else                                
       write(ucf,'(''$maxsym off'')')
    endif
    if(ctrl%sumdeg) then
       write(ucf,'(''$sumdeg on'')')
    else                                
       write(ucf,'(''$sumdeg off'')')
    endif
    if(ctrl%print_pol) then
       write(ucf,'(''$print_pol on'')')
    else                                
       write(ucf,'(''$print_pol off'')')
    endif
    if(ctrl%print_gaten) then
       write(ucf,'(''$print_gaten on'')')
    else                                
       write(ucf,'(''$print_gaten off'')')
    endif
    if(ctrl%print_rcs) then
       write(ucf,'(''$print_rcs on'')')
    else                                
       write(ucf,'(''$print_rcs off'')')
    endif
    if(ctrl%print_vroa) then
       write(ucf,'(''$print_vroa on'')')
    else                                
       write(ucf,'(''$print_vroa off'')')
    endif
    if(ctrl%print_transrot) then
       write(ucf,'(''$print_transrot on'')')
    else
       write(ucf,'(''$print_transrot off'')')
    endif
    if(ctrl%print_zpe) then
       write(ucf,'(''$print_zpe on'')')
    else
       write(ucf,'(''$print_zpe off'')')
    endif
    if(ctrl%print_thermo) then
       write(ucf,'(''$print_thermo on'')')
    else
       write(ucf,'(''$print_thermo off'')')
    endif
    if (ctrl%vroaew.eq.0) then
      write(ucf,'(a,i10)') '$ramanew ',ctrl%ramanew
      if(associated(ctrl%dynfreq)) then
        write(ucf,'(''   '',10f16.8)') (ctrl%dynfreq(i),&
            &i=1,ubound(ctrl%dynfreq,1))
      endif
    end if
    if (ctrl%ramanew.eq.0) then
      write(ucf,'(a,i10)') '$vroaew ',ctrl%vroaew
      if(associated(ctrl%dynfreq)) then
        write(ucf,'(''   '',10f16.8)') (ctrl%dynfreq(i),&
            &i=1,ubound(ctrl%dynfreq,1))
      else if (ctrl%vroaew.eq.1) then
        write(ucf,'(a)') '     0.0D0'
      endif
    end if
!MR $dynfreq flag is kept for simple read out in read_restart !
!MR   although this introduces redundant information in snf_control
    if((ctrl%ramanew.gt.0.or.ctrl%vroaew.gt.0).and.&
      &associated(ctrl%dynfreq)) then
       write(ucf,'(''$dynfreq '',10f16.8)') (ctrl%dynfreq(i),&
            &i=1,ubound(ctrl%dynfreq,1))
    endif
    if(associated(ctrl%dal_ctrl)) then
      if(ctrl%dal_ctrl%dalmw.gt.0) then
        write(ucf,'(a,i10)') '$dalmw ',ctrl%dal_ctrl%dalmw
      end if
    end if
    write(ucf,'(''$total electronic energy '',f20.10)') ctrl%elec_energy
    write(ucf,'(''$total spin '',f20.10)') ctrl%spin
    do i=1,ubound(molecule,1)
       if(molecule(i)%changem) then
          write(ucf,'(''$changemass'',1x,i5,f24.16)') i,molecule(i)%mass
       end if
    end do
    write(ucf,'(a9,i4)') '$nummodes ',ctrl%nummodes
    if(ctrl%nummodes.ne.0) then
      natoms = ubound(molecule,1)
      do i = 1,allmodes
        selectmode=ctrl%selectmode(i)
        if(selectmode.eq.1) then
          do j = 1,natoms
            if(j.eq.1) then
              write(ucf,'(f11.4,3f11.4)') &
  &           ctrl%freqin(i), ctrl%nmodein(i,j), &
  &           ctrl%nmodein(i,j+1),ctrl%nmodein(i,j+2)
            else
              write(ucf,'(11x,3f11.4)') ctrl%nmodein(i,(j-1)*3+1), &
  &           ctrl%nmodein(i,(j-1)*3+2),ctrl%nmodein(i,(j-1)*3+3)
            end if
          end do
        end if
      end do
    write(ucf,'(a10,i4)') '$disptype ',ctrl%disptype
    end if
    write(ucf,'(''$end'')')
    close(ucf)
    write(out,*)
    write(out,'(''  Created new snf control file : '',A)') trim(adjustl(ctrl%control))
  end subroutine write_control

  !************************************************************************

  subroutine copy_ctrl_to_gauss(ctrl)
    !----------------------------------------------------------------
    !
    ! written by C. Herrmann, Uni Jena 2005
    !----------------------------------------------------------------
    implicit none
    integer :: i
    type(type_ctrl),intent(inout) :: ctrl
    if(ctrl%ramanew.ge.1) then 
      ctrl%gauss_ctrl%ramanew=ctrl%ramanew
      if(associated(ctrl%dynfreq)) then
        if(associated(ctrl%gauss_ctrl%dynfreq)) deallocate(ctrl%gauss_ctrl%dynfreq)
        allocate(ctrl%gauss_ctrl%dynfreq(ubound(ctrl%dynfreq,1)))
        do i=1,ubound(ctrl%dynfreq,1)
          ctrl%gauss_ctrl%dynfreq(i)=ctrl%dynfreq(i)
        end do
      end if
    else
       ctrl%gauss_ctrl%ramanew=0
    endif
    ctrl%gauss_ctrl%calcno=ctrl%calcno
  end subroutine copy_ctrl_to_gauss
  !************************************************************************

  subroutine copy_ctrl_to_adf(ctrl)
    !----------------------------------------------------------------
    !
    ! written by C. Herrmann, Uni Jena 2005
    !----------------------------------------------------------------
    implicit none
    integer :: i
    type(type_ctrl),intent(inout) :: ctrl
    if(ctrl%ramanew.ge.1) then 
      ctrl%adf_ctrl%ramanew=ctrl%ramanew
      if(associated(ctrl%dynfreq)) then
        if(associated(ctrl%adf_ctrl%dynfreq)) deallocate(ctrl%adf_ctrl%dynfreq)
        allocate(ctrl%adf_ctrl%dynfreq(ubound(ctrl%dynfreq,1)))
        do i=1,ubound(ctrl%dynfreq,1)
          ctrl%adf_ctrl%dynfreq(i)=ctrl%dynfreq(i)
        end do
      end if
    else
       ctrl%adf_ctrl%ramanew=0
    endif
    ctrl%adf_ctrl%calcno=ctrl%calcno
  end subroutine copy_ctrl_to_adf

  !************************************************************************

  subroutine copy_ctrl_to_dal(ctrl)
    !----------------------------------------------------------------
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    ! modified by M. Reiher, Uni Erlangen, June 2003
    !----------------------------------------------------------------
    implicit none
    integer :: i
    type(type_ctrl),intent(inout) :: ctrl
    if(ctrl%ramanew.ge.1) then 
      ctrl%dal_ctrl%ramanew=ctrl%ramanew
      if(associated(ctrl%dynfreq)) then
        if(associated(ctrl%dal_ctrl%dynfreq)) deallocate(ctrl%dal_ctrl%dynfreq)
        allocate(ctrl%dal_ctrl%dynfreq(ubound(ctrl%dynfreq,1)))
        do i=1,ubound(ctrl%dynfreq,1)
          ctrl%dal_ctrl%dynfreq(i)=ctrl%dynfreq(i)
        end do
      end if
    else
       ctrl%dal_ctrl%ramanew=0
    endif
    if(ctrl%vroaew.ge.1) then
      ctrl%dal_ctrl%vroaew=ctrl%vroaew
      if(associated(ctrl%dynfreq)) then
        if(associated(ctrl%dal_ctrl%dynfreq)) deallocate(ctrl%dal_ctrl%dynfreq)
        allocate(ctrl%dal_ctrl%dynfreq(ubound(ctrl%dynfreq,1)))
        do i=1,ubound(ctrl%dynfreq,1)
          ctrl%dal_ctrl%dynfreq(i)=ctrl%dynfreq(i)
        end do
      end if
    else
       ctrl%dal_ctrl%vroaew=0
    endif
    ctrl%dal_ctrl%calcno=ctrl%calcno
    ctrl%dal_ctrl%dal_func=ctrl%dal_func
!    ctrl%dal_ctrl%udalinp=ctrl%udalinp
  end subroutine copy_ctrl_to_dal

  !************************************************************************

  subroutine copy_ctrl_to_tm(ctrl)
    !----------------------------------------------------------------
    !
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    ! re-written by J. Neugebauer, Uni Erlangen, 2003
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    !----------------------------------------------------------------
    implicit none
    integer :: i
    type(type_ctrl),intent(inout) :: ctrl
    ctrl%tm_ctrl%ramanew=ctrl%ramanew
    if(associated(ctrl%dynfreq)) then
      if(associated(ctrl%tm_ctrl%dynfreq)) deallocate(ctrl%tm_ctrl%dynfreq)
      allocate(ctrl%tm_ctrl%dynfreq(ubound(ctrl%dynfreq,1)))
      do i=1,ubound(ctrl%dynfreq,1)
        ctrl%tm_ctrl%dynfreq(i)=ctrl%dynfreq(i)
      end do
    end if
    ctrl%tm_ctrl%calcno=ctrl%calcno
  end subroutine copy_ctrl_to_tm

  !************************************************************************

  subroutine copy_tm_to_ctrl(ctrl)
    !----------------------------------------------------------------
    !
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    ! re-written by J. Neugebauer, Uni Erlangen, 2003
    !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    integer i
    type(type_ctrl),intent(inout) :: ctrl
    ctrl%ramanew=ctrl%tm_ctrl%ramanew
    if(associated(ctrl%tm_ctrl%dynfreq)) then
      if(associated(ctrl%dynfreq)) deallocate(ctrl%dynfreq)
      allocate(ctrl%dynfreq(ubound(ctrl%tm_ctrl%dynfreq,1)))
      do i=1,ubound(ctrl%dynfreq,1)
        ctrl%dynfreq(i)=ctrl%tm_ctrl%dynfreq(i)
      end do
      ctrl%ramanew=ubound(ctrl%dynfreq,1)
    end if
    if(ctrl%tm_ctrl%symmetry .ne. 'c1 ') then
       ctrl%symmetry=ctrl%tm_ctrl%symmetry
    endif
  end subroutine copy_tm_to_ctrl

  !************************************************************************

  subroutine print_ctrl(ctrl,unit)
    !----------------------------------------------------------------
    ! output routine
    !
    ! original version by Carsten Kind, Uni Erlangen, 2000-2001
    ! re-written by J. Neugebauer, Uni Erlangen, 2003
    ! modified by M. Reiher, Uni Erlangen, June 2003
    ! modified by C. Herrmann, Uni Bonn, March 2005
    !----------------------------------------------------------------
    implicit none
    type(type_ctrl),intent(in) :: ctrl
    integer,optional :: unit
    integer :: exopt
    character(len=10)  :: spinstate
    integer :: out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    if(ctrl%progno.eq.1) then
      if(ctrl%tm_ctrl%smult.eq.1) then
        spinstate='singlet'
      else if(ctrl%tm_ctrl%smult.eq.3) then
        spinstate='triplet'
      else
        spinstate='[ERROR]'
      end if
      if(exopt.eq.0) exopt=ctrl%tm_ctrl%rpast
    end if
    write(out,*)
    write(out,'('' SNF Control file information'')')
    write(out,'('' ----------------------------'')')
    write(out,'(''   snf control file          : '',A)') trim(adjustl(ctrl%control))
    write(out,'(''   title                     : '',A)') trim(adjustl(ctrl%title))
    write(out,'(''   snf restart file          : '',A)') trim(adjustl(ctrl%restart))
    write(out,'(''   restart file format vers. :'',F6.2)') dble(ctrl%version)/100.D0
    write(out,'(''   symmetry                  :   '',A3)') ctrl%symmetry
    write(out,'(''   Number of atoms           :'',I6)') &
                                             & ctrl%nvar/3
    write(out,'(''   Number of steps           :'',I6)') ctrl%numderiv*2
    write(out,'(''   distortion [bohr]         : '',F6.3)') ctrl%cstep
    if(ctrl%progno.eq.1) then
      write(out,'(''   SCF convergence criterion :'',I6)') ctrl%tm_ctrl%scfconv
      if(ctrl%tm_ctrl%rpast.gt.0) then
         write(out,*)
         write(out,'(''   Excited state calculation: '')')
         write(out,'(2x,a,i7)')' no. of exc. states opt. with egrad    : ',&
   &     ctrl%tm_ctrl%rpast
         write(out,'(2x,a,i7)')' state no. calculated in SNF           : ',exopt
         write(out,'(2x,2a)')' spin state of excited states          : ',spinstate
         write(out,'(2x,2a)')' type of exc. state calculations       : ',ctrl%tm_ctrl%extype
         write(out,*)
      end if
    end if
    if(ctrl%ramanew.gt.0) then
       write(out,'(''   Raman intensities switched on'')')
       if(associated(ctrl%dynfreq)) write(out,'(''   Dynamic polarizabilities will be &
         &used'')')
    end if
    if(ctrl%vroaew.gt.0) then
       write(out,'(''   VROA calculation switched on'')')
       if(associated(ctrl%dynfreq)) write(out,'(''   Freq.-dependent properties will be &
         &used'')')
    end if
    if(ctrl%nummodes.gt.0) then
       write(out,'(''   intensities only calculation switched on'')')
       write(out,'(2x,a,i7)')' no. of normal modes read in           : ',&
  &    ctrl%nummodes
    end if
    if(ctrl%print_gaussian) &
         & write(out,'(''   print gaussian fake output'')')
    if(ctrl%print_mopac) then
     write(out,'(''   print mopac fake output'')')
      if(ctrl%print_mopxmol) &
        write(out,'(''   for use in xmol'')')
    end if
    if(ctrl%print_xmol) &
         & write(out,'(''   print xmol input file'')')
    if(ctrl%print_symmetry) &
         & write(out,'(''   print detailed symmetry-information'')')
    if(ctrl%print_irred_at) &
         & write(out,'(''   print information about symmetry redundant atoms'')')
    if(ctrl%print_irred_dist) &
         & write(out,'(''   print information about symmetry redundant&
         & distortions'')')
    if(ctrl%print_cioslowski) &
         & write(out,'(''   print cioslowski charge decomposition'')')
    if(ctrl%print_force) &
         & write(out,'(''   print force constant matrix'')')
    if(ctrl%print_hessian) &
         & write(out,'(''   print cartesian hessian'')')
    if(ctrl%print_dip) &
         & write(out,'(''   print cartesian dipol derivates'')')
    if(ctrl%print_pol) &
         & write(out,'(''   print cartesian polarizability derivates'')')
    if(ctrl%print_gaten) &
         & write(out,'(''   print cartesian G and A tensor derivates'')')
    if(ctrl%print_rcs) &
         & write(out,'(''   print raman scattering cross sections'')')
    if(ctrl%print_vroa) &
         & write(out,'(''   print VROA results'')')
    if(ctrl%print_transrot) &
         & write(out,'(''   print transl. and rot. contributions'')')
    if(ctrl%print_zpe) &
         & write(out,'(''   print  zeropoint energy '')')
    if(ctrl%print_thermo) &
         & write(out,'(''   print thermodynamics'')')
    write(out,'(''   Plot range for spectra  [cm^-1]       : '',I4,''-'',I4)')&
         & int(Ctrl%minfreq),int(Ctrl%maxfreq)
    write(out,'(''   half-width for peaks [cm^-1]          :'',I4)') int(ctrl%width)
    write(out,'(''   peak type in spectra plots            :'',a)') ctrl%peaktype
    write(out,'(''   Scaling factor for frequencies        :'',f24.12)') ctrl%freqscale    
  end subroutine print_ctrl

  !************************************************************************

  subroutine check_ctrl(ctrl1,ctrl2,unit)
    implicit none
    !----------------------------------------------------------------
    ! compare ctrl1 (established from restart file) and ctrl2
    ! (established from snf_control / control)
    !
    ! written by Carsten Kind, Uni Erlangen, 2000-2001
    !----------------------------------------------------------------
    type(type_ctrl),intent(in) :: ctrl1,ctrl2
    integer,optional :: unit
    integer :: out
    if(present(unit)) then
       out=unit
    else
       out=6
    endif
    if (ctrl1%version/10.gt.ctrl2%version/10) then
       write (out,*) 'ERROR: incompatible versions '
       write (out,*) '   --> version on restart file :',ctrl1%version
       write (out,*) '   --> current program version :',ctrl2%version
       stop 'incompatible restart file'
    endif
!JN: Turbomole control file is not read in SNF run => mode no.
!    is only determined by restart file
!    if (ctrl1%mode .ne. ctrl2%mode) then
!       write (out,*) 'ERROR: <control> file has been changed:'
!       write (out,*) '   --> mode on restart file :',ctrl1%mode
!       write (out,*) '   --> current program mode :',ctrl2%mode
!       stop 'incompatible restart file'
!    endif
    if (dabs(ctrl1%cstep-ctrl2%cstep).gt.1.d-6) then
       write (out,*) 'ERROR: cstep has been changed:'
       write (out,*) '   --> cstep on restart file     :',ctrl1%cstep
       write (out,*) '   --> current program cstepmode :',ctrl2%cstep
       stop 'incompatible restart file'
    endif
    if (ctrl1%numderiv.ne.ctrl2%numderiv) then
       write (out,*) 'ERROR: numderiv has been changed !'
       write (out,*) '   --> numderiv on restart file     :',ctrl1%numderiv
       write (out,*) '   --> current program numderiv :',ctrl2%numderiv
       stop 'incompatible restart file'
    endif
!JN: See above
!    if (iabs(ctrl1%scfconv-ctrl2%scfconv).gt.0) then
!       write (out,*) 'ERROR: scfconv has been changed:'
!       write (out,*) '   --> scfconv on restart file     :',ctrl1%scfconv
!       write (out,*) '   --> current program scfconv :',ctrl2%scfconv
!       stop 'incompatible restart file'
!    endif
  end subroutine check_ctrl

  !************************************************************************

  subroutine destroy_control(ctrl)
    implicit none
    type(type_ctrl),intent(inout) :: ctrl
    if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
    if(associated(ctrl%dal_ctrl)) deallocate(ctrl%dal_ctrl)
  end subroutine destroy_control

  !************************************************************************

        subroutine get_spin(dir,spin,out)
!
!       written by Markus Reiher, University of Erlangen, Nov. 2001
!
        use snf_tools,only     : string2real,clean
        implicit none
        character(len=*)     :: dir
        real(double)         :: spin,si,thres
        character(len=132)   :: buf
        character(len=256)   :: errbuf
        character(len=20)    :: cdumb
        integer              :: out,i
        logical              :: ex
        parameter(thres=0.25d0)
        inquire(file=dir,exist=ex)
        if(.not.ex) goto 999
        open(unit=45,file=dir,status='unknown')
10      read(45,'(a)',err=999,end=999) buf
        if (index(buf(1:32),'<S*S>').ne.0) then
          call clean(cdumb)
          cdumb(1:20)=buf(7:26)
          spin=string2real(cdumb)
!	*** <S*S> approx S(S+1) => extract S
	  do i=1,10
            si=0.5d0*dble(i)
 	    if (abs(si*(si+1.0d0)-spin).le.thres) then
	      spin=si
              close(45)
              return
            endif
          enddo
	  write(errbuf,'(3a)') ' total spin > 5 ?!,&
            &total spin will be set to zero.'
          errbuf=trim(adjustl(errbuf))
	  call serror('get_spin',0,errbuf,out) 
	  close(45)
	  return
        endif
        goto 10
999     errbuf=''
        write(errbuf,'(3a)') 'Reading total spin from file ''',&
          &trim(adjustl(dir)),''' failed.## Total spin will be set to zero.'
        errbuf=trim(adjustl(errbuf))
        call serror('get_spin',0,errbuf,out)
        close(45)
        return
        end subroutine get_spin

  !************************************************************************

        subroutine get_energy(dir,e,out)
!
!       written by Markus Reiher, University of Erlangen, Aug. 2001
!       modified by J. Neugebauer, University of Erlangen, Sep. 2001
!
        use snf_tools,only     : string2real,clean
        implicit none
        character(len=*)     :: dir
        double precision     :: e
        character(len=132)   :: buf
        character(len=256)   :: errbuf
        character(len=20)    :: cdumb
        integer              :: out
        logical              :: ex
        inquire(file=dir,exist=ex)
        if(.not.ex) goto 999
        open(unit=45,file=dir,status='unknown')
10      read(45,'(a)',err=999,end=999) buf
        if (buf(1:32).eq.'                 |  total energy') then
          call clean(cdumb)
          cdumb(1:20)=buf(index(buf,'=')+1:index(buf,'=')+19)
          e=string2real(cdumb)
          close(45)
          return
        endif
        goto 10
999     errbuf=''
        write(errbuf,'(3a)') 'Reading total energy from file ''',&
          &trim(adjustl(dir)),''' failed.## Total energy will be set to zero.' 
        errbuf=trim(adjustl(errbuf))
        call serror('get_energy',0,errbuf,out)
        close(45)
        return
        end subroutine get_energy 

  !************************************************************************
  
        subroutine get_energy_mp2(dir,e,out)
!
!       written by Markus Reiher, University of Erlangen, Aug. 2001
!       modified by J. Neugebauer, University of Erlangen, Sep. 2001
!
        use snf_tools,only     : string2real,clean
        implicit none
        character(len=*)  :: dir
        double precision  :: e
        character(len=132):: buf
        character(len=256):: errbuf
        character(len=22) :: cdumb
        character(len=14) :: cdumc
        integer out
        logical           :: ex
        inquire(file=dir,exist=ex)
        if(.not.ex) goto 999
        open(unit=45,file=dir,status='unknown')
10      read(45,'(a)',err=999,end=999) buf
        cdumc(1:14)=buf(21:35)
        if (cdumc.eq.'total        :') then
          call clean(cdumb)
          cdumb(1:22)=buf(index(buf,':')+1:index(buf,':')+21)
          e=string2real(cdumb)
          close(45)
          return
        endif
        goto 10
999     errbuf=''
        write(errbuf,'(3a)') 'Reading total energy from file ''',&
          &trim(adjustl(dir)),''' failed.## Total energy will be set to zero.' 
        errbuf=trim(adjustl(errbuf))
        call serror('get_energy_mp2',0,errbuf,out)
        close(45)
        return
        end subroutine get_energy_mp2

 
  !************************************************************************  


       SUBROUTINE get_dalenergy(escf,name,mode)
!
! read CCS/CCSD/CC2 energy and gradient from file <name>
! in case of DALTON Coupled Cluster (or MP2) calculations
!
! written by J. Neugebauer, Uni Erlangen, 2002.
!
      use snf_tools, only : string2real,split
      implicit none
      INTEGER lu,irc,mxitem,mode
      parameter(mxitem=10)
      CHARACTER*(*) name
      real(double) escf
      CHARACTER*128 rec
      CHARACTER*128 item(mxitem),errbuf
      escf=0.D0
      lu=1
      OPEN (UNIT=lu,FILE=name,STATUS='UNKNOWN',IOSTAT=irc)
      IF (irc.NE.0) RETURN
90    READ (lu,'(A)',END=99) rec
      CALL split(rec,item,mxitem)
      IF(mode.eq.4.and.(index(rec,'Final    HF energy:').ne.0)) goto 91
      IF (item(1).EQ.'Total'.and.item(3).eq.'energy:'.and. &
     & (item(2).eq.'CC2'.or.item(2).eq.'CCSD'.or.&
     & (item(2).eq.'MP2'.and.(mode.eq.2.or.mode.eq.8)))) GOTO 91
      GOTO 90
91    CONTINUE
      escf=string2real(item(4))
      GOTO 90
 
99    if(escf.gt.-1.0d-10) then
       errbuf=''
        write(errbuf,'(3a)') 'Reading total energy from file ''',&
          &trim(adjustl(name)),''' failed.## Total energy will be set to zero.' 
        errbuf=trim(adjustl(errbuf))
        call serror('get_dalenergy',0,errbuf)
      end if
      CLOSE(lu)
      END subroutine get_dalenergy
 
  !************************************************************************  

      SUBROUTINE get_dalescf(escf,name,mode)
!
! read SCF energy and gradient from file <name>
! in case of DALTON SCF/MCSCF calculations
!
! written by J. Neugebauer, Uni Erlangen, 2002.
!
      implicit none
      INTEGER lu,irc,mxitem,mode
      parameter(mxitem=10)
      CHARACTER*(*) name
      real(double) escf
      CHARACTER*128 rec
      CHARACTER*128 item(mxitem),errbuf
      escf=0.D0
      lu=1
      OPEN (UNIT=lu,FILE=name,STATUS='UNKNOWN',IOSTAT=irc)
      IF (irc.NE.0) RETURN
90    READ (lu,'(A)',END=99) rec
      IF(mode.eq.6) then
        IF (index(rec,'Final MCSCF energy:').ne.0) then
          read(rec(25:),*) escf
          goto 90
        END IF
      ELSE
        IF (index(rec,'Final    HF energy:').ne.0) then
          read(rec(25:),*) escf
          goto 90
        END IF
      END IF
      GOTO 90
 
99    if(escf.gt.-1.0d-10) then
       errbuf=''
        write(errbuf,'(3a)') 'Reading total energy from file ''',&
          &trim(adjustl(name)),''' failed.## Total energy will be set to zero.' 
        errbuf=trim(adjustl(errbuf))
        call serror('get_dalescf',0,errbuf)
      end if
      CLOSE(lu)
      RETURN
      END subroutine get_dalescf

 
  !************************************************************************  

end module snf_control

