
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_dalton
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param
  use snf_rbas,     only: read_tm_bas, type_basis
  
  private
  public :: type_dal_ctrl, init_dal_ctrl,write_dal_ctrl
  
  type type_dal_ctrl
    real(double),pointer :: dynfreq(:) ! angular frequencies for dyn. pol.
    logical  :: direct             ! integral direct DALTON calculation
    logical  :: udalinp            ! use original DALTON input files
    character(len=132) :: coord    ! coord file in DALTON format
    character(len=132) :: control  ! control file for DALTON
    character(len=132) :: mos      ! use DALTON MOs form file <mos>
    character(len=7) :: dal_func   ! the functional for a DALTON DFT run
    integer :: progno,  &          ! program type
         &     calcno,       &     ! calculation type
         &     ramanew,        &   ! no. of polarizability sets
         &     vroaew,         &   ! no. of frequencies for propertz calc (vroa is kept orthogonal to raman)
         &     hfocc,          &   ! HF occupation in case of MCSCF
         &     cas,            &   ! CAS SPACE
         &     caselec,        &   ! ELECTRONS (IN CAS SPACE)
         &     nstate,         &   ! STATE NO. nstate will be optimized in MCSCF
         &     inactive,       &   ! No. of inactive orbitals (CASSCF)
         &     dalmw,          &   ! dalton work memory
         &     multiplicity        ! SPIN MULTIPLICITY for MCSCF
  end type type_dal_ctrl

  !*********************************************************************
 
contains
 
  !*********************************************************************
   
  subroutine init_dal_ctrl(ctrl)
    !----------------------------------------------------------------
    ! initialize default ctrl values for dalton.inp
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    ! extended by M. Reiher, Uni Erlangen, June 2003
    ! extended by M. Reiher, Uni Bonn, April 2004
    !---------------------------------------------------------------- 
    implicit none
    type(type_dal_ctrl), intent(out) :: ctrl
    nullify(ctrl%dynfreq)
    ctrl%udalinp=.true.
    ctrl%progno=2
    ctrl%calcno=1
    ctrl%ramanew=0
    ctrl%vroaew=0
    ctrl%direct=.false.
    ctrl%coord='molecule.mol'
    ctrl%control='molecule.dal'
    ctrl%dal_func='       '
    ctrl%hfocc=0
    ctrl%cas=0
    ctrl%caselec=0
    ctrl%nstate=0
    ctrl%inactive=0
    ctrl%multiplicity=1
  end subroutine init_dal_ctrl
 
  !*********************************************************************
   
  subroutine write_dal_ctrl(ctrl,molecule)
    !---------------------------------------------------------------- 
    ! writes the file ctrl%control. The file molecule.mol will be 
    ! created, which is not necessary for
    ! the Snfdc calculation (this file has to be created for every
    ! slave process, but not for the master process).
    ! It is also possible to supply MO start vectors in the 
    ! DALTON.MOPUN format from file <ctrl%mos>
    ! Note: keyword '.DIRECT' might be introduced after 
    ! '**DALTON INPUT' to perform integral direct calculations.
    ! However, these calculations can be slower than integral non-direct 
    ! calculations!
    !
    ! written by J. Neugebauer, Uni Erlangen, 2001
    ! supplemented for VROA keywords by M. Reiher, Uni. Erlangen, 2003
    ! modified by C. Herrmann, Uni Bonn, March 2005
    !---------------------------------------------------------------- 
      use snf_molecule, only: type_atom
      implicit none
      type(type_dal_ctrl) :: ctrl
      type(type_atom), intent(in) :: molecule(:)
      type(type_basis), pointer :: basis(:)
      integer info,system,inout,i,natoms
      integer,pointer :: ic(:)
      real(double),pointer :: co(:,:)
      character(len=132) buf
      character(len=10) calctype
      logical ex,soppa
 
      write(*,*) 
      write(*,*) '======================================='
      write(*,*) '=== CREATING DALTON INPUT FILES ... ==='
      write(*,*) '======================================='
      write(*,*) 

      soppa=.false.
      natoms=ubound(molecule,1)
      if(.not.ctrl%udalinp) then
        allocate(ic(natoms),co(3,natoms))
        do i=1,ubound(molecule,1)
          ic(i)=molecule(i)%nucl_charge
          co(:,i)=molecule(i)%cartcoord(:)
        end do
        call read_tm_bas(basis)
        call wdalton(natoms,co,ic,basis)
        deallocate(ic,co)
        call rmfile('molecule.dal',1)
      else
        call bfile('molecule.dal')
        call bfile('molecule.mol')
      end if

! special treatment for SOPPA calculations
      if(ctrl%calcno.eq.8) then
        soppa=.true.
        ctrl%calcno=2
      end if

      inout=25
      call rmfile('snf.tmp',1)
      open(inout,file='snf.tmp')
      write(inout,'(a)') '**DALTON INPUT'
! VROA if required (for SCF, MCSCF and DFT)
      if(ctrl%vroaew.ge.1) then
        write(inout,'(a)') '.WALK'
        write(inout,'(a)') '.MAX IT'
        write(inout,'(a)') ' 0     '
      else
        write(inout,'(a)') '.RUN WAVE FUNCTION'
      end if
      if(ctrl%direct) write(inout,'(a)') '.DIRECT'
      if(ctrl%calcno.eq.1.or.ctrl%calcno.eq.6.or.ctrl%calcno.eq.666) then
! special treatment in case of VROA
! (DO NOT CHANGE THE ORDER OF ANY OF THESE LINES)
      if(ctrl%vroaew.ge.1) then
        write(inout,'(a)') '*WALK'
!CMR: the following is apparentlz not needed if .MOLGRA is given
!!CMR0404
!          write(inout,'(a)') '.NUMHES'
!!CMR0404
        write(inout,'(a)') '.NUMERI'
        write(inout,'(a)') '.PRINT'
        write(inout,'(a)') ' 1 '
        write(inout,'(a)') '.DISPLA'
        write(inout,'(a)') ' 0.001'
        write(inout,'(a)') '**START'
        write(inout,'(a)') '.VROA'
      else
        write(inout,'(a)') '.RUN PROPERTIES'
        write(inout,'(a)') '**PROPERTIES'
      end if
!
      write(inout,'(a)') '.MOLGRA'
      if(ctrl%ramanew.ge.1) then
        if(.not.associated(ctrl%dynfreq)) then
          write(inout,'(a)') '.POLARI'
        else
!CMR300404
!CMR300404 The following line was removed because it is no longer supported in dalton
!CMR300404
!          write(inout,'(a)') '.ABALNR'
!CMR300404
          write(inout,'(a)') '*ABALNR'
          write(inout,'(a)') '.ALFA  '
          write(inout,'(a)') '.FREQUE'
!CMR: the following is apparentlz not needed if .MOLGRA is given
!!CMR0404
!          write(inout,'(a)') '.NUMHES'
!!CMR0404
          write(inout,'(i2)') ubound(ctrl%dynfreq)
          write(inout,'(10f18.10)') (ctrl%dynfreq(i),i=1,&
           &ubound(ctrl%dynfreq,1))
        end if
      end if
! VROA for SCF, MCSCF and DFT if required
      if(ctrl%vroaew.ge.1) then
        if(.not.associated(ctrl%dynfreq)) then
          write(inout,'(a)') '.POLARI'
        else
!CMR300404
!CMR300404 The following line was removed because it is no longer supported in dalton
!CMR300404
!          write(inout,'(a)') '.ABALNR'
!CMR300404
          write(inout,'(a)') '*ABALNR'
!MR For VROA, .ALFA must never be switched on 
!   - otherwise the G and A tensors are not calculated!
!MR          write(inout,'(a)') '.ALFA  '
!CMR300404
!CMR300404 the following does not appear to make sense
!CMR300404    because the MO coeffciients are obtained with
!CMR300404    10**-6 so the perturbed coefficients cannot be
!CMR300404    obtained with higher accuracy and no conv. would be obtained
!CMR300404
!          write(inout,'(a)') '.THRESH '
!          write(inout,'(a)') '5.0D-8 '
!CMR300404
          write(inout,'(a)') '.FREQUE'
          write(inout,'(i2)') ubound(ctrl%dynfreq)
          write(inout,'(10f18.10)') (ctrl%dynfreq(i),i=1,&
           &ubound(ctrl%dynfreq,1))
        end if
      end if
!
      if(.not.ctrl%direct) then
        write(inout,'(a)') '**INTEGRALS'
        write(inout,'(a)') '.NOSUP'
      end if
      write(inout,'(a)') '**WAVE FUNCTIONS'
!CMR0404
      if (ctrl%calcno.eq.666) then
        write(inout,'(a)') '.DFT'
        write(inout,'(a)') ctrl%dal_func
!CMR0404
      else
        write(inout,'(a)') '.HF'
      end if
        if(ctrl%calcno.eq.6) then
          call readcasin(ctrl%hfocc,ctrl%multiplicity,ctrl%inactive, &
     & ctrl%cas,ctrl%caselec,ctrl%nstate)
          call casinput(ctrl%hfocc,ctrl%multiplicity,ctrl%inactive, &
     & ctrl%cas,ctrl%caselec,ctrl%nstate)
          write(inout,'(a)') '.MCSCF'
          write(inout,'(a)') '*OPTIMIZATION'
          write(inout,'(a)') '.MAX MACRO ITERATIONS'
          write(inout,'(a)') '40'
          write(inout,'(a)') '.MAX MICRO ITERATIONS'
          write(inout,'(a)') '30'
          if(ctrl%nstate.ne.0) then
            write(inout,'(a)') '.STATE'
            write(inout,'(i2)') ctrl%nstate+1
          end if
!CMR280404 the following line was needed by old DALTON versions:
!CMR280404          write(inout,'(a)') '*HF INPUT'
!CMR280404 the new name is now "SCF INPUT", however it is kept
!CMR280404 the old way in case of ab initio calculations in order to
!CMR280404 have top down compability
	  if (ctrl%calcno.eq.666) then
	    write(inout,'(a)') '*SCF INPUT'
	  else
	    write(inout,'(a)') '*HF INPUT'
	  end if
          write(inout,'(a)') '*HF INPUT'
!MR: the following should perhaps included for all calculations:
          if(ctrl%vroaew.ge.1) then
            write(inout,'(a)') '.THRESH '
            write(inout,'(a)') '1.0D-8 '
          end if
          write(inout,'(a)') '.HF OCCUPATION'
          write(inout,'(i2)') ctrl%hfocc
          write(inout,'(a)') '*CONFIGURATION INPUT'
          write(inout,'(a)') '.SYMMETRY'
          write(inout,'(a)') '1'
          write(inout,'(a)') '.SPIN MULTIPLICITY'
          write(inout,'(i2)') ctrl%multiplicity
          write(inout,'(a)') '.INACTIVE'
          write(inout,'(i2)') ctrl%inactive
          write(inout,'(a)') '.CAS SPACE'
          write(inout,'(i2)') ctrl%cas
          write(inout,'(a)') '.ELECTRONS'
          write(inout,'(i2)') ctrl%caselec
        end if
      else if(ctrl%calcno.ge.2.and.ctrl%calcno.le.5) then
        !--------------------------------------------------
        ! coupled cluster (or MP2 using CC module)
        !--------------------------------------------------
        call calc_type(ctrl%progno,ctrl%calcno,calctype)
        write(inout,'(a)') '**INTEGRALS'
        write(inout,'(a)') '.DIPLEN'
        write(inout,'(a)') '.DEROVL'
        write(inout,'(a)') '.DERHAM'
        if(.not.ctrl%direct) write(inout,'(a)') '.NOSUP'
        write(inout,'(a)') '**WAVE FUNCTIONS'
        write(inout,'(a)') '.CC'
        write(inout,'(a)') '*CC INPUT'
        write(inout,'(a)') '.'//trim(adjustl(calctype))
        write(inout,'(a)') '*CCFOP'
        write(inout,'(a)') '.DIPMOM'
        if(soppa) goto 111
        if(ctrl%ramanew.ge.1) then
          write(inout,'(a)') '*CCLR'
          write(inout,'(a)') '.DIPOLE'
          write(inout,'(a)') '.FREQUE'
          if(associated(ctrl%dynfreq)) then
            write(inout,'(i2)') ubound(ctrl%dynfreq)
            write(inout,'(10f18.10)') (ctrl%dynfreq(i),i=1,&
             &ubound(ctrl%dynfreq,1))
          else
            write(inout,'(a)') '1'
            write(inout,'(a)') '0.0d0'
          end if
        end if
111     write(inout,'(a)') '*DERIVATIVES'
      else if(ctrl%calcno.eq.7) then
        !--------------------------------------------------
        ! SCF using RESPONSE module of DALTON rather than
        ! ABACUS
        !--------------------------------------------------
        if(ctrl%ramanew.ge.1) write(inout,'(a)') '.RUN RESPONSE'
        write(inout,'(a)') '.RUN PROPERTIES'
        write(inout,'(a)') '**PROPERTIES'
        write(inout,'(a)') '.MOLGRA'
        if(ctrl%ramanew.ge.1) then
          write(inout,'(a)') '**RESPONSE'
          write(inout,'(a)') '*LINEAR'
          write(inout,'(a)') '.DIPLEN'
          write(inout,'(a)') '.FREQUE'
          if(.not.associated(ctrl%dynfreq)) then
            write(inout,'(a)') '1'
            write(inout,'(f18.10)') 0.0d0
          else
            write(inout,'(i2)') ubound(ctrl%dynfreq)
            write(inout,'(10f18.10)') (ctrl%dynfreq(i),i=1,&
             &ubound(ctrl%dynfreq,1))
          end if
        end if
        write(inout,'(a)') '**WAVE FUNCTIONS'
        write(inout,'(a)') '.HF'
      end if
      write(inout,'(a)') '*ORBITAL INPUT'
      write(inout,'(a)') '.NOSUPSYM'
      !--------------------------------------------------------------
      ! supply start MOs, if desired
      !--------------------------------------------------------------
      close(inout)
      info=system('mv snf.tmp molecule.dal')
!CH280305      open(inout,file='molecule.dal')
!CH280305      do
!CH280305        read(inout,'(a)',end=800) buf
!CH280305      end do
!CH280305800   write(inout,'(a)') '*END OF INPUT'
!CH280305      close(inout)
!CH280305 using the code above resulted in the following error message:
!CH280305 A WRITE statement cannot be executed after an endfile record is read (unit=25).
!CH280305 --> replaced it by the following line:
      info=system('echo *END OF INPUT >> molecule.dal')
      if(soppa) then
        ctrl%calcno=8
        call rmfile('snf.tmp',1)
        open(inout,file='molecule.dal.B')
        write(inout,'(a)') '**DALTON INPUT'
        write(inout,'(a)') '.RUN WAVE FUNCTION'
        write(inout,'(a)') '.RUN RESPONSE'
        if(ctrl%ramanew.ge.1) then
          write(inout,'(a)') '**RESPONSE'
          write(inout,'(a)') '.SOPPA'
          write(inout,'(a)') '*LINEAR'
          write(inout,'(a)') '.DIPLEN'
          write(inout,'(a)') '.FREQUE'
          if(.not.associated(ctrl%dynfreq)) then
            write(inout,'(a)') '1'
            write(inout,'(f18.10)') 0.0d0
          else
            write(inout,'(i2)') ubound(ctrl%dynfreq)
            write(inout,'(10f18.10)') (ctrl%dynfreq(i),i=1,&
             &ubound(ctrl%dynfreq,1))
          end if
        end if
        write(inout,'(a)') '**WAVE FUNCTIONS'
        write(inout,'(a)') '.HF'
        write(inout,'(a)') '.MP2'
        write(inout,'(a)') '*ORBITAL INPUT'
        write(inout,'(a)') '.NOSUPSYM'
        write(inout,'(a)') '*END OF INPUT'
        close(inout)
      end if
      write(*,*) 
      write(*,*) '======================================'
      write(*,*) '=== CREATED ALL DALTON INPUT FILES ==='
      write(*,*) '=== (press <return> to continue)   ==='
      write(*,*) '======================================'
      write(*,*) 
      read(*,*)
      return
900   call serror('write_dal_ctrl',2,'Error while preparing&
       & DALTON input file.')
 
  end subroutine write_dal_ctrl
 
  !*********************************************************************

      subroutine casinput(hfocc,mult,inac,cas,ecas,nstate)
      !---------------------------------------------------------------- 
      ! interactive input of data required for DALTON CASSCF 
      ! calculation
      !
      ! written by J. Neugebauer, Uni Erlangen, 2002
      !---------------------------------------------------------------- 
      implicit none
      integer :: hfocc,mult,inac,cas,ecas,nstate,i
      character(len=80) :: zeile
      logical :: valid
 
      valid=.false.
100   do while(.not.valid)
        write(*,*)
        write(*,*)
        write(*,*) ' Additional input is necessary for CASSCF calculations.'
        write(*,*) ' Syntax: parameter <parameter_value>'
        write(*,*) 
        write(*,*) ' Please enter the following parameters:'
        write(*,*) 
        write(*,'(a)') '    parameter: meaning                  : current setting'
        write(*,'(a)') '    -----------------------------------------------------'
        write(*,'(a,i3)') '    hfoc     : no. of occ. HF orbitals  : ',hfocc
        write(*,'(a,i3)') '    inac     : no. of inactive orbitals : ',inac
        write(*,'(a,i3)') '    ocas     : no. of orbitals in CAS   : ',cas
        write(*,'(a,i3)') '    ecas     : no. of electrons in CAS  : ',ecas
        write(*,'(a,i3)') '    mult     : spin multiplicity        : ',mult
        write(*,'(a,i3)') '    stno     : no. of state to be calc. : ',nstate
        write(*,'(a,i3)') '               (0=ground, n=nth excited state)'
        write(*,*)
        write(*,'(a)') '    >return< : quit this menu'
        write(*,*)
        read(*,'(a)') zeile
        zeile=trim(adjustl(zeile))
        if(len_trim(zeile).eq.0) then
          valid=.true.
        else 
          if(len(zeile).le.4) goto 100
          read(zeile(5:),*,err=100,end=100) i
          select case(zeile(1:4))
           case ('hfoc')
             hfocc=i
           case ('mult')
             mult=i
           case ('inac')
             inac=i
           case ('ocas')
             cas=i
           case ('ecas')
             ecas=i
           case ('stno')
             nstate=i
           case default
          end select
        end if
      end do      
      end subroutine casinput
   
  !*********************************************************************

      subroutine wdalton(natoms,co,ic,basis)
!
! write DALTON input file "molecule.mol" to avoid based call
!
! written by J. Neugebauer, Uni Erlangen, 2002
!
      implicit none
      integer natoms,i,j,ih,ltd,ic(natoms),kcharge,ntyp,iat,mxsh
      parameter(mxsh=1500)
      type(type_basis), pointer :: basis(:)
      integer lam,nco(10)
      real*8 co(3,natoms)
      character*2 symtxt
      character*1 charm
      parameter(ih=1)
      open(unit=ih,file='molecule.mol')
      WRITE (IH,'(A)') 'INTGRL'
      WRITE (IH,'(A)') 'DALTON input converted from TURBOMOLE'
      WRITE (IH,'(A)') '====================================='
      kcharge=0
      call molchar(natoms,ic,kcharge)
!**
! determine number of different atom types
!**
      ntyp=1
      do 75 j=2,natoms
        do i=1,j-1
          if(ic(i).eq.ic(j)) goto 75
        end do
        ntyp=ntyp+1
75    continue
      symtxt=' 0'
      charm=' '
      WRITE (IH,8800) charm,ntyp,kcharge,symtxt
8800  FORMAT(A1,I4,I3,A2)
      ntyp=0
      do 71 iat=1,natoms
        do i=1,iat-1
          if(ic(iat).eq.ic(i)) goto 71
        end do
        do i=1,ubound(basis,1)
          if(ic(iat).eq.basis(i)%attype) ntyp=i
        end do
        lam=ubound(basis(ntyp)%basset,1)
        write(43,*) 'IAT: ',iat,' ntyp: ',ntyp,' LAM: ',lam
        do i=1,lam
          nco(i)=1
        end do
        call wdalgeo(ih,iat,lam,nco,symtxt,ic,natoms,co)
        call wrgcdal(ih,lam,basis(ntyp))
71    continue
      close(ih)
      end subroutine wdalton
 
 
 
 
!************************************************************************
 
      SUBROUTINE wdalgeo(ih,iat,lam,nco,stxt,ic,natoms,co)
!
! write geometry record
!
!JN
! modified wmolgeo routine, taken from based (by B.A. Hess)
! modifications by J. Neugebauer, University of Erlangen, 2002
!JN
      use snf_molecule
      implicit none
      integer natoms
      INTEGER ih,iat,lam,nco(lam),ic(natoms)
      real*8 co(3,natoms)
! local
      CHARACTER*1 kaset
      CHARACTER*2 stxt
      INTEGER irc,nont,i,j,k
 
      kaset=' '
 
! determine number of centres with equal nuclear charge number
      nont=0
      DO j=1,natoms
        IF(ic(j).eq.ic(iat)) nont=nont+1
      END DO
      WRITE (ih,11) kaset,dble(ic(iat)),nont,lam,(nco(j),j=1,lam)
11    FORMAT (A1,F9.0,12I5)
 
! this atom
      WRITE (ih,12) find_element(ic(iat)),(co(j,iat),j=1,3)
! the other atoms of this type
      DO k=iat+1,natoms
       IF(ic(iat).eq.ic(k)) then
        WRITE (ih,12) find_element(ic(iat)),(co(j,k),j=1,3)
       END IF
      ENDDO
12    FORMAT (A2,2X,3F20.15)
      RETURN
      END subroutine wdalgeo
 
 
!************************************************************************
 
 
      subroutine molchar(natoms,ic,mcharge)
!
! calculate molecular charge from occupation numbers in control
!
! written by J. Neugebauer, Uni Erlangen, 2002
!
      use snf_molecule, only: gotm21
      implicit none
      integer mxsh,natoms,nmo,nel,ic(natoms),mcharge,lfn
      integer i,ih2
      parameter(ih2=2,mxsh=1500)
      integer nocc(mxsh)
      integer elec
      character*4 irrep(mxsh)
      character*132 buf
      nmo=0
      mcharge=0
      do i=1,natoms
        mcharge=mcharge+ic(i)
      end do
      nel=0
      open(unit=ih2,file='control')
1     continue
      read(ih2,'(A)',end=999) buf
9     continue
      elec=0
      IF (buf(1:15).EQ.'$closed shells ') elec=2
      IF (buf(1:13).EQ.'$open shells ') elec=1
      IF (buf(1:14).EQ.'$alpha shells ') elec=1
      IF (buf(1:13).EQ.'$beta shells ') elec=1
      if(elec.le.0) goto 1
      call gotm21(ih2,irrep,nmo,nocc,mxsh,buf)
      do i=1,nmo
        call upper(irrep(i))
        if(index(irrep(i),'E').ne.0) then
          nel=nel+2*int(elec*nocc(i)+.5d0)
        else if(index(irrep(i),'A').ne.0) then
          nel=nel+int(elec*nocc(i)+.5d0)
        else if(index(irrep(i),'B').ne.0) then
          nel=nel+int(elec*nocc(i)+.5d0)
        else if(index(irrep(i),'T').ne.0) then
          nel=nel+3*int(elec*nocc(i)+.5d0)
        else if(index(irrep(i),'F').ne.0) then
          nel=nel+4*int(elec*nocc(i)+.5d0)
        else if(index(irrep(i),'H').ne.0) then
          nel=nel+5*int(elec*nocc(i)+.5d0)
        else if(index(irrep(i),'G').ne.0) then
          nel=nel+4*int(elec*nocc(i)+.5d0)
        end if
      end do
      goto 9
999   close(ih2)
      write(43,'(3(a,i4,2x))') ' molchar | nuclear charge = ',mcharge, &
     & ', no. el. = ',nel,' molecular charge = ',mcharge-nel
      mcharge=mcharge-nel
      end subroutine molchar
 
 
!************************************************************************
 
      subroutine wrgcdal(ih,lam,abasis)
!
! write generalized contraction (basis in DALTON format)
!
! based on a subroutine from program "based" by B. A. Hess.
!
! written by J. Neugebauer, University of Erlangen, 2002
!
      implicit none
      type(type_basis) :: abasis
      integer i,j,maxexp,maxcon,ip,ih,lam,k,l
      parameter(maxexp=200,maxcon=40)
      real(double) :: coef(maxexp,maxcon),expo(maxexp)
      do i=1,lam
        do j=1,maxexp
          do k=1,maxcon
            coef(j,k)=0.0d0
          end do
          expo(j)=0.0d0
        end do
        ip=0
        do j=1,ubound(abasis%basset(i)%con,1)
          do k=1,ubound(abasis%basset(i)%con(j)%expo,1)
            do l=1,ip
              if(abs(abasis%basset(i)%con(j)%expo(k)-expo(l)).lt.1.0d-8) then
                coef(l,j)=abasis%basset(i)%con(j)%coef(k)
                goto 90
              end if
            end do
            ip=ip+1
            expo(ip)=abasis%basset(i)%con(j)%expo(k)
            coef(ip,j)=abasis%basset(i)%con(j)%coef(k)
90          continue
          end do
        end do
        call wrexbdal(ih,ip,ubound(abasis%basset(i)%con,1),maxexp,&
     &   maxcon,coef,expo)
      end do
      end subroutine wrgcdal

!************************************************************************
 
        subroutine wrexbdal(ih,ip,ic,maxexp,maxcon,co,ex)
!
! write single gen. contracted basis function
!
! based on a subroutine from program "based" by B. A. Hess.
! written by J. Neugebauer, University of Erlangen, 2002
!
        implicit none
        integer ih,ip,ic,maxexp,maxcon,j,k
        real(double) :: co(maxexp,maxcon),ex(maxexp)
        character(len=72) coform

        write(ih,'(a1,i4,i5)') 'H',ip,ic
        do k=1,ip
          if(ic.le.3) then
            coform='(4f20.10)'
          else
            coform='(4f20.10/,(20x,3f20.10))'
          end if
          write(ih,coform) ex(k),(co(k,j),j=1,ic)
        end do
        end subroutine wrexbdal

!************************************************************************

      subroutine readcasin(hfocc,mult,inac,cas,ecas,nstate)
      !---------------------------------------------------------------- 
      ! try to read CAS input information from file molecule.dal
      !
      ! written by J. Neugebauer, Uni Erlangen, 2002
      !---------------------------------------------------------------- 
      implicit none
      integer :: hfocc,mult,inac,cas,ecas,nstate,i,lu
      character(len=132) :: zeile,fname
      logical :: valid

      fname='molecule.dal'
      inquire(file=fname,exist=valid)
      if(.not.valid) then
        fname='molecule.dal.bak'
        inquire(file=fname,exist=valid)
        if(.not.valid) return
      end if
      valid=.false.
      open(lu,file=fname)
      do while(.not.valid)
        read(lu,'(A)',err=999,end=999) zeile
        if(index(zeile,'.STATE').ne.0) then
          read(lu,*,err=999,end=999) nstate
          nstate=nstate-1
        else if(index(zeile,'.SPIN MULTIPLICITY').ne.0) then
          read(lu,*,err=999,end=999) mult
        else if(index(zeile,'.INACTIVE').ne.0) then
          read(lu,*,err=999,end=999) inac
        else if(index(zeile,'.CAS SPACE').ne.0) then
          read(lu,*,err=999,end=999) cas
        else if(index(zeile,'.ELECTRONS').ne.0) then
          read(lu,*,err=999,end=999) ecas
        else if(index(zeile,'.HF OCC').ne.0) then
          read(lu,*,err=999,end=999) hfocc
        else if(index(zeile,'*END OF INPUT').ne.0) then
          valid=.true.
        end if
      end do
999   close(lu)
      end subroutine readcasin

end module snf_dalton
