
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_vroa
  use f90_kind
  use snf_error

  private
  public :: create_aten_dxyz,print_aten_dxyz
  public :: create_aten_dq,create_aten_dq_intonly,calcvroa
  public :: expand_gten,symm_gten

  !************************************************************************

contains

  !************************************************************************

  subroutine create_aten_dxyz(polar_dxyz,pols,distortion)
    implicit none
    !----------------------------------------------------------------
    ! calculation of first derivative of a tensor with arbitrary number
    ! of tensor components according to 
    ! the following 3-point formula:
    ! 
    !  d alpha_ij     pols_ij(xi+e(xi)) - pols_ij(xi-e(xi))  
    ! ------------ = ---------------------------------------   
    !    dxi                     2*e(xi) 
    !
    ! a 5- and 7-point central difference formulae are also implemented
    !           
    ! Note: the units are atomic units and the step size is therefore
    !       required in bohr!
    !
    ! written by M. Reiher, Uni Erlangen, June 2003
    !----------------------------------------------------------------
    real(double),intent(inout) :: polar_dxyz(:,:)
    real(double),intent(in) :: pols(:,:,:)
    real(double),intent(in) :: distortion(:)
    integer :: idum,i
  end subroutine create_aten_dxyz
  !************************************************************************


  subroutine create_aten_dq_intonly(polar_dqi,polar,nummodes,normmodes,cstep,numderiv,natoms,molecule)
    use snf_molecule
    implicit none
    !----------------------------------------------------------------
    ! calculation off first derivative of A-tensor with
    ! respect to non mass-weighted normal coordinates using the
    ! following formula:
    !
    !  d polar    polar(qi+e(qi) - polar(qi-e(qi))
    ! -------- = ---------------------------------
    !    dqi                 2*distortion
    !
    ! we use one distortion value for all normal modes
    ! Note: the units are atomic units and the step size is therefore
    !       required in bohr!
    !
    ! written by C. Herrmann, Uni Bonn 2005,
    ! based on create_polar_dxyz, written by Carsten Kind and
    ! Markus Reiher, J. Neugebauer, Uni Erlangen, 1999-2002
    !----------------------------------------------------------------
    real(double),intent(inout):: polar_dqi(:,:)
    real(double),intent(in):: polar(:,:,:)
    real(double) :: cstep
    integer :: i,j ,natoms,nummodes,numderiv
    real(double) :: normmodes(natoms*3,natoms*3)
    real(double) :: zthres, sum, red_mass(nummodes)
    type(type_atom),pointer :: molecule(:)
    red_mass(:) = 0.d0
  end subroutine create_aten_dq_intonly


  !************************************************************************

  subroutine print_aten_dxyz(polar_dxyz,filename)
    implicit none
    !----------------------------------------------------------------
    ! output routine
    !
    ! written by Markus Reiher, Uni Erlangen, June 2003
    !----------------------------------------------------------------
    character*(*) :: filename
    integer :: i,k
    real(double),intent(in) :: polar_dxyz(:,:)

    open(unit=45,file=filename,status='unknown')
    write(45,*) 'cartesian tensor derivatives'
    write(45,'(3d18.10)') ((polar_dxyz(i,k),i=1,ubound(polar_dxyz,1)),k=1,ubound(polar_dxyz,2))
    close (45)
  end subroutine print_aten_dxyz

  !************************************************************************

  subroutine create_aten_dq(polar_dxyz,norm_modes,molecule,polar_dq)
    !----------------------------------------------------------------
    ! written by M. Reiher, Uni Erlangen, June 2003
    !
    ! Note: Cartesian tensor derivatives are given in a.u.
    !----------------------------------------------------------------
    use snf_molecule, only: type_atom
    implicit none
    real(double),intent(in) :: polar_dxyz(:,:),norm_modes(:,:)
    type(type_atom), pointer :: molecule(:)
    real(double),intent(inout):: polar_dq(:,:)
    integer :: i,j,k,nvar,irrep_dim
  end subroutine create_aten_dq

  !************************************************************************

  subroutine expand_gten(gten_dxyz,representation,symm_index,char_table)
    use snf_char_table, only : type_char_table
    use snf_symmat,only : symel_mat
    !----------------------------------------------------------------
    ! copy second-rank 3x3 tensor derivatives calculated 
    ! on irreducible atoms to the
    ! reducible atoms. this is done by multiplying symmetry representations
    ! in the pol basis from the left and symmetry representations in the
    ! derivate basis from the right with the tensor component derivatives
    ! and selecting only the required columns.
    !
    ! written by Markus Reiher, Uni Erlangen, June 2003
    !----------------------------------------------------------------
    implicit none
    type(type_char_table), intent(in) :: char_table
    real(double),intent(inout) :: gten_dxyz(:,:)
    real(double),pointer :: rot_polar(:,:),expanded_polar(:,:),polar_tmp(:,:)
    real(double),intent(in) :: representation(:,:,:)
    integer, intent(in) :: symm_index(:)
    integer :: s,i,offset,nvar,counter,j,k,natoms
    real(double) :: pol(3,3),transpol(3,3)
  end subroutine expand_gten

  !************************************************************************

  subroutine symm_gten(gten_dxyz,rep,char_table)
    use snf_char_table, only : type_char_table
    use snf_symmat,only : symel_mat
    !----------------------------------------------------------------
    ! this subroutine transforms the  
    ! second-rank 3x3 tensor component derivatives
    ! according to all symmetry operations of the point group.
    ! all resulting matrices are added and then diveded by the
    ! number of symmetry operations. a symmetry check is performed.
    !
    ! written by Markus Reiher, Uni Erlangen, June 2003
    !----------------------------------------------------------------
    implicit none
    type(type_char_table), intent(in) :: char_table
    real(double),intent(inout) :: gten_dxyz(:,:)
    real(double),pointer :: rot_polar(:,:),polar_tmp(:,:),tmpmat(:,:)
    real(double),intent(in) :: rep(:,:,:)
    real(double) :: errsum
    integer :: s,i,nvar,counter,j,k
    character(len=60) :: errbuff
    real(double) :: pol(3,3),transpol(3,3)
  end subroutine symm_gten

  !************************************************************************

  subroutine calcvroa(freqs,polar9_dq,gtenao_dq,gtenlao_dq,aten_dq,&
                      &NCORD,NUMMOD,NFRVAL,MXFR,FRVAL,LUPRI)
    implicit none
    !----------------------------------------------------------------
    ! driver for DALTON VROA evaluation routines
    !
    ! written by Markus Reiher, Uni Erlangen, June 2003
    !----------------------------------------------------------------

    real(double),intent(in) :: freqs(:),polar9_dq(:,:,:),gtenao_dq(:,:,:),gtenlao_dq(:,:,:),aten_dq(:,:,:)
    real(double),intent(in) :: FRVAL(:)
    real(double),pointer    :: ROAAFQ(:,:,:,:),ROAGNQ(:,:,:,:),ROAGLQ(:,:,:,:),ROAAQ(:,:,:,:,:)
    real(double),pointer    ::  dumROAAQ(:,:,:,:,:)
    real(double),pointer    :: BAL(:),ALMN(:),CONVER(:),BOLTZ(:)
    real(double),pointer    :: BGML(:),BGMNL(:),BA(:),GMMNL(:),GMMNNL(:),RMINN (:),DPOLN (:),DZL(:),&
                              &DZNL(:),DXL(:),DXNL(:),D0L(:),D0NL(:),DBL(:),DBNL(:),CZL(:),CZNL(:),&
                              &CXL(:),CXNL(:),C0L(:),C0NL(:),CBL(:),CBNL(:),FREQ(:)
    integer NFRVAL,NUMMOD,MXFR,LUPRI,NCORD,i,j,idum,k,l,idum2,XTKAYS
    parameter(XTKAYS=219474.6313710D0)
!... store derivatives in format of the DALTON routine
  end subroutine calcvroa

  !************************************************************************

end module snf_vroa
