      subroutine hessianmw(hessian,natoms,ic)
*-----------------------------------------------------------------------
* mass-weighting of the cartesian hessian matrix
*
* written by J. Neugebauer, Uni Erlangen, 2002
*-----------------------------------------------------------------------
      implicit none
      integer i,j,natoms
      integer ic(natoms)
      real*8 hessian(3*natoms,3*natoms),find_mass,mi,mj
      external find_mass
      do i=1,3*natoms
        mi=sqrt(find_mass(ic((i+2)/3)))
        do j=1,i
          mj=sqrt(find_mass(ic((j+2)/3)))
          hessian(j,i)=hessian(j,i)/mi/mj
          hessian(i,j)=hessian(j,i)
        end do
      end do
      return
      end

***********************************************************************

      subroutine hessianumw(hessian,natoms,ic)
*-----------------------------------------------------------------------
* de-mass-weighting of the mass-weighted cartesian hessian matrix
*
* written by J. Neugebauer, Uni Erlangen, 2002
*-----------------------------------------------------------------------
      implicit none
      integer i,j,natoms
      integer ic(natoms)
      real*8 hessian(3*natoms,3*natoms),find_mass,mi,mj
      external find_mass
      do i=1,3*natoms
        mi=sqrt(find_mass(ic((i+2)/3)))
        do j=1,i
          mj=sqrt(find_mass(ic((j+2)/3)))
          hessian(j,i)=hessian(j,i)*mi*mj
          hessian(i,j)=hessian(j,i)
        end do
      end do
      return
      end

***********************************************************************

      subroutine hessian_m2nm(hessianc,natoms,normmodes)
*-----------------------------------------------------------------------
* unitary transformation of mass-weighted cartesian hessian
* matrix to hessian matrix in the basis of normal coordinates of the 
* equilibrium structure
*
* [might be inefficient for large molecules, because for every hessian
* it is necessary to allocate memory and deallocate it afterwards.]
*
* written by J. Neugebauer, Uni Erlangen, 2002
*-----------------------------------------------------------------------
      include 'defs.h'
      include 'objects.h'
      integer natoms,nvar,ptmp
      real*8 hessianc(3*natoms,3*natoms),normmodes(3*natoms,3*natoms)
      nvar=3*natoms
      call memmgr(memgdw,'tmpmat  ',s8,ptmp,nvar*nvar)
      call dgemm('N','N',nvar,nvar,nvar,1.D0,hessianc,nvar,normmodes,
     * nvar,0.d0,s8(ptmp),nvar)
      call dgemm('T','N',nvar,nvar,nvar,1.D0,normmodes,nvar,s8(ptmp),
     * nvar,0.d0,hessianc,nvar)
      call memmgr(memfdw,'tmpmat  ',s8,ptmp,nvar*nvar)
      return
      end

***********************************************************************

      subroutine hessian_nm2m(hessianc,natoms,normmodes)
*-----------------------------------------------------------------------
* unitary transformation of the hessian matrix in the basis of 
* normal coordinates of the equilibrium structure to the mass-weighted
* cartesian hessian
*
* written by J. Neugebauer, Uni Erlangen, 2002
*-----------------------------------------------------------------------
      include 'defs.h'
      include 'objects.h'
      integer natoms,nvar,ptmp
      real*8 hessianc(3*natoms,3*natoms),normmodes(3*natoms,3*natoms)
      nvar=3*natoms
      call memmgr(memgdw,'tmpmat  ',s8,ptmp,nvar*nvar)
      call dgemm('N','T',nvar,nvar,nvar,1.D0,hessianc,nvar,normmodes,
     * nvar,0.d0,s8(ptmp),nvar)
      call dgemm('N','N',nvar,nvar,nvar,1.D0,normmodes,nvar,s8(ptmp),
     * nvar,0.d0,hessianc,nvar)
      call memmgr(memfdw,'tmpmat  ',s8,ptmp,nvar*nvar)
      return
      end

***********************************************************************

      subroutine calcfreq(hessianq,natoms,freq)
*----------------------------------------------------------------------
* calculation of harmonic vibrational wavenumbers from the diagonalized
* mass-weighted hessian matrix
*
* units: 1/cm
* 
* written by J. Neugebauer, Uni Erlangen, 2002
* based on the corresponding F90 subroutine by C. Kind
*----------------------------------------------------------------------
      implicit none
      include 'fconstants.h'
      integer natoms,i
      real*8 hessianq(3*natoms,3*natoms),freq(3*natoms),conv,wavn

********
*    conversion factors:
*   
*    frequency[1/s]   = 1/(2*pi)*sqrt(d^2 E/ dq^2)
*    wavenumber[1/cm] = 1/c * frequency
*   
*    (d^2 E/ dq^2) must be given in [J/(m^2 kg)], but is
*    given in [hartree/(bohr^2*amu)] due to mass-weighting in [amu],
*
*    => conversion factor for (d^2 E/ dq^2):
*    hartree/(amu*bohr^2) [J/(m^2 kg)]
*
*    wavenumbers should be given in [1/cm] rather than in [1/m]
*    => factor 1/1.d02 arises 
********

      conv=hartree/(amu*bohr*bohr)

C*********
C      write(*,'(a,e24.14)') 'conv = ',conv
C      wavn=conv/(2.d0*pi)/(2.d0*pi)/(1.d02*speedoflight)/
C     * (1.d02*speedoflight)
C      write(*,'(a,e24.14)') 'wavnum ',wavn
C*********
C NOTE: wavn is in TURBOMOLE (in rotvib.f) wavnum=2.6436411815482d+07
C       here wavn=2.6424608118585d+07
C       => frequencies calculated from TURBOMOLE cartesian hessian
C          differ by sqrt(wavnum/wavn)=1.0002233217419d0
C*********

      do i=1,3*natoms
        freq(i)=hessianq(i,i)*conv
        if(hessianq(i,i).lt.0.0d0) then
          freq(i)=-(1.d0/(2.d0*pi)*sqrt(-freq(i))/(1.d02*speedoflight))
        else
          freq(i)=1.d0/(2.d0*pi)*sqrt(freq(i))/(1.d02*speedoflight)
C          write(*,'(a,2f24.14)')'freq ',sqrt(hessianq(i,i)*wavn),freq(i)
C          write(*,'(a,2f24.14)')'---- ',sqrt(hessianq(i,i)*conv/
C     * (1.d02*speedoflight)/(1.d02*speedoflight)/(2.d0*pi)/(2.d0*pi))
        end if
      end do
      return
      end

***********************************************************************

      function fqfc(freq)
*
* this function has the same purpose like calcqfc.
* either fcalcqfc or calcqfc should be removed in future.
*
* written by J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      real*8 fqfc,qfc,freq
      call calcqfc(qfc,freq)
      fqfc=qfc
      end

***********************************************************************

      subroutine calcqfc(qfc,freq)
*----------------------------------------------------------------------
* calculation of a single harmonic (quadratic) force constant (i.e., a
* diagonal element of the force constant matrix) from the 
* vibrational wavenumber 
*
* units: wavenumber input in 1/cm, 
*        force constant output in hartree/bohr^2/amu
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------
      implicit none
      include 'fconstants.h'
      integer i
      real*8 qfc,freq,conv

********
*    conversion factors:
*   
*    frequency[1/s]   = 1/(2*pi)*sqrt(d^2 E/ dq^2)
*    wavenumber[1/cm] = 1/c * frequency
*   
*    (d^2 E/ dq^2) must be given in [J/(m^2 kg)], but is
*    given in [hartree/(bohr^2*amu)] due to mass-weighting in [amu],
*
*    => conversion factor for (d^2 E/ dq^2):
*    hartree/(amu*bohr^2) [J/(m^2 kg)]
*
*    wavenumbers should be given in [1/cm] rather than in [1/m]
*    => factor 1/1.d02 arises 
********

      conv=hartree/(amu*bohr**2)
      qfc=freq*(1.d02*speedoflight)*(2.d0*pi)
      qfc=qfc*qfc
      if(freq.lt.0.0d0) qfc=-qfc
      qfc=qfc/conv
      return
      end

***********************************************************************

      subroutine dhessdi(hess,dhi,cstepq)
*
* calculate 1st derivative of hessian in the basis of mass-weighted
* normal modes with respect to mass-weighted normal coordinates
*
* written by J. Neugebauer, Uni Erlangen, 2002
*
      include 'defs.h'
      include 'param.h'
      integer i,j
      real*8 hess(nvar,nvar,naus),dhi(nvar,nvar),cstepq
      
      if(naus.eq.2) then
        do j=1,nvar
          do i=1,nvar
            dhi(i,j)=1.0d0/(2.0d0*cstepq)*(hess(i,j,1)-hess(i,j,2))
          end do 
        end do
      else if(naus.eq.4) then
        do j=1,nvar
          do i=1,nvar
            dhi(i,j)=1.0d0/(24.0d0*cstepq)*(2.0d0*hess(i,j,4)
     *      -16.0d0*hess(i,j,3)+16.0d0*hess(i,j,2)-2.0d0*hess(i,j,1))
          end do
        end do
      else if(naus.eq.6) then
        do j=1,nvar
          do i=1,nvar
            dhi(i,j)=1.0d0/(720.0d0*cstepq)*(
     *       -12.0d0*hess(i,j,6)+108.0d0*hess(i,j,5)
     *      -540.0d0*hess(i,j,4)+540.0d0*hess(i,j,3)
     *       -108.0d0*hess(i,j,2)+12.0d0*hess(i,j,1))
          end do
        end do
      else
        write(ftape,*) 'ERROR! naus = ',naus,' is not implemented !'
        stop
      end if
      end

***********************************************************************

      subroutine ddhessdii(hess,hesseq,dhii,cstepq)
*
* calculate 2nd derivative of hessian in the basis of mass-weighted
* normal modes with respect to mass-weighted normal coordinates
* 
* only elements d^2 hess/di^2 are calculated !
*
* written by J. Neugebauer, Uni Erlangen, 2002
*
      include 'defs.h'
      include 'param.h'
      integer i,j
      real*8 hess(nvar,nvar,naus),dhii(nvar,nvar),cstepq
      real*8 hesseq(nvar,nvar)
      
      if(naus.eq.2) then
        do j=1,nvar
          do i=1,nvar
            dhii(i,j)=1.0d0/cstepq/cstepq*(hess(i,j,2)
     *         +hess(i,j,1)-2.0d0*hesseq(i,j))
          end do 
        end do
      else if(naus.eq.4) then
        do j=1,nvar
          do i=1,nvar
            dhii(i,j)=1.0d0/(12.0d0*cstepq*cstepq)*(-hess(i,j,4)+
     *       16.0d0*hess(i,j,3)-30.0d0*hesseq(i,j)+16.0d0*hess(i,j,2)
     *       -hess(i,j,1))
          end do
        end do
      else if(naus.eq.6) then
        do j=1,nvar
          do i=1,nvar
            dhii(i,j)=1.0d0/(360.0d0*cstepq*cstepq)*(
     *         4.0d0*hess(i,j,6)-54.0d0*hess(i,j,5)
     *      +540.0d0*hess(i,j,4)+540.0d0*hess(i,j,3)
     *       -54.0d0*hess(i,j,2)+4.0d0*hess(i,j,1)-980.0d0*hesseq(i,j))
          end do
        end do
      else
        write(ftape,*) 'ERROR! naus = ',naus,' is not implemented !'
        stop
      end if
      end

***********************************************************************

      function qfc2freq(qfc)
*----------------------------------------------------------------------
* calculation of a single harmonic vibrational wavenumber from the 
* force constant qfc
*
* units: 1/cm
* 
* written by J. Neugebauer, Uni Erlangen, 2002
* based on the corresponding F90 subroutine by C. Kind
*----------------------------------------------------------------------
      implicit none
      include 'fconstants.h'
      integer i
      real*8 qfc2freq,qfc,conv

********
*    conversion factors:
*   
*    frequency[1/s]   = 1/(2*pi)*sqrt(d^2 E/ dq^2)
*    wavenumber[1/cm] = 1/c * frequency
*   
*    (d^2 E/ dq^2) must be given in [J/(m^2 kg)], but is
*    given in [hartree/(bohr^2*amu)] due to mass-weighting in [amu],
*
*    => conversion factor for (d^2 E/ dq^2):
*    hartree/(amu*bohr^2) [J/(m^2 kg)]
*
*    wavenumbers should be given in [1/cm] rather than in [1/m]
*    => factor 1/1.d02 arises 
********

      conv=hartree/(amu*bohr*bohr)
      qfc2freq=qfc*conv
      if(qfc.lt.0.0d0) then
        qfc2freq=-(1.d0/(2.d0*pi)*sqrt(-qfc2freq)/(1.d02*speedoflight))
      else
        qfc2freq=1.d0/(2.d0*pi)*sqrt(qfc2freq)/(1.d02*speedoflight)
      end if
      end
